@@ -28,8 +28,8 @@
  * SUCH DAMAGE.
  */
 
-#ifndef LXQT_OSX_KEYCHAIN_H_H
-#define LXQT_OSX_KEYCHAIN_H_H
+#ifndef LXQT_SECRET_SERVICE_H
+#define LXQT_SECRET_SERVICE_H
 
 #include "lxqt_wallet.h"
 
@@ -40,63 +40,67 @@
 
 #include <memory>
 
-class QWidget ;
+class QWidget;
 
-namespace LXQt{
+namespace LXQt
+{
 
-namespace Wallet{
+namespace Wallet
+{
 
 class osxKeyChain : public LXQt::Wallet::Wallet
 {
 public:
-	osxKeyChain() ;
-	~osxKeyChain() ;
+    osxKeyChain();
+    ~osxKeyChain();
 
-	void open( const QString& walletName,
-		   const QString& applicationName,
-		   std::function< void( bool ) >,
-		   QWidget * = nullptr,
-		   const QString& password = QString(),
-		   const QString& displayApplicationName = QString() ) ;
+    void open(const QString &walletName,
+              const QString &applicationName,
+              std::function< void(bool) >,
+              QWidget * = nullptr,
+              const QString &password = QString(),
+              const QString &displayApplicationName = QString());
 
-	bool open( const QString& walletName,
-		   const QString& applicationName,
-		   QWidget * = nullptr,
-		   const QString& password = QString(),
-		   const QString& displayApplicationName = QString() ) ;
+    bool open(const QString &walletName,
+	      const QString &applicationName,
+	      QWidget * = nullptr,
+	      const QString &password = QString(),
+	      const QString &displayApplicationName = QString());
 
-	bool addKey( const QString& key,const QByteArray& value ) ;
-	bool opened( void ) ;
+    bool addKey(const QString &key, const QByteArray &value);
+    bool opened(void);
 
-	QByteArray readValue( const QString& key ) ;
+    QByteArray readValue(const QString &key);
 
-	QVector< std::pair< QString,QByteArray > > readAllKeyValues( void ) ;
+    QVector<std::pair<QString, QByteArray>> readAllKeyValues(void);
 
-	QStringList readAllKeys( void ) ;
-	QStringList managedWalletList( void ) ;
+    QStringList readAllKeys(void);
+    QStringList managedWalletList(void);
 
-	QString storagePath( void ) ;
-	QString localDefaultWalletName( void ) ;
-	QString networkDefaultWalletName( void ) ;
+    QString storagePath(void);
+    QString localDefaultWalletName(void);
+    QString networkDefaultWalletName(void);
 
-	void deleteKey( const QString& key ) ;
-	void closeWallet( bool ) ;
-	void changeWalletPassWord( const QString& walletName,
-				   const QString& applicationName = QString(),
-				   std::function< void( bool ) > = []( bool e ){ Q_UNUSED( e ) } ) ;
-	void setImage( const QIcon& ) ;
+    void deleteKey(const QString &key);
+    void closeWallet(bool);
+    void changeWalletPassWord(const QString &walletName,
+                              const QString &applicationName = QString(),
+                              std::function< void(bool) > = [](bool e) { Q_UNUSED(e); });
+    void setImage(const QIcon &);
 
-	int walletSize( void )  ;
+    int walletSize(void) ;
 
-	LXQt::Wallet::BackEnd backEnd( void ) ;
-	QObject * qObject( void ) ;
+    void log(std::function<void(QString)>);
+
+    LXQt::Wallet::BackEnd backEnd(void);
+    QObject *qObject(void);
 private:
-	bool m_opened = false ;
-	QByteArray m_walletName ;
+    bool m_opened = false ;
+    QByteArray m_walletName ;
 };
 
 }
 
 }
 
-#endif // LXQT_OSX_KEYCHAIN_H
+#endif // LXQT_KWALLET_H
