@@ -26,11 +26,28 @@ public:
 
 	void updateOptions( engines::engine::cmdArgsList&,bool ) const override ;
 
-	engines::engine::status errorCode( const QString& e,int s ) const override ;
+	engines::engine::status errorCode( const QString& e,const QString& err,int s ) const override ;
 
 	void GUICreateOptions( const engines::engine::createGUIOptions& ) const override ;
 
 	void GUIMountOptions( const engines::engine::mountGUIOptions& ) const override ;
+
+	const QStringList& windowsUnmountCommand() const override ;
+
+	QByteArray prepareBackend() const override ;
+
+	args command( const QByteArray& password,
+		      const engines::engine::cmdArgsList& args,
+		      bool create ) const override ;
 private:
+#ifdef Q_OS_WIN
+#else
 	const engines::versionGreaterOrEqual m_version_has_error_codes ;
+#endif
+	QString m_sirikaliCppcryptfsExe ;
+	QString m_cppcryptfsctl ;
+	QString m_cppcryptfs ;
+	QStringList m_windowsUnmountCommand ;
+	mutable QString m_cppcryptfsPid ;
+	QString getcppcryptfsPid() const ;
 } ;
