@@ -1,4 +1,4 @@
-﻿/*
+/*
  *
  *  Copyright (c) 2012-2015
  *  name : Francis Banyikwa
@@ -204,7 +204,7 @@ keyDialog::keyDialog( QWidget * parent,secrets& s,
 
 	this->setUpInitUI() ;
 
-	this->setUpVolumeProperties( { favorites::instance().unknown() } ) ;
+	this->setUpVolumeProperties( {} ) ;
 
 	m_ui->lineEditMountPoint->setText( QString() ) ;
 
@@ -217,16 +217,6 @@ void keyDialog::setUpInitUI()
 
 	m_keyType.keyOptions( m_ui->cbKeyType,this,&keyDialog::keyTypeChanged ) ;
 
-	if( utility::platformIsWindows() ){
-
-		/*
-		 * We are diabling this functionality on windows for now
-		 * its currently not possible to open a volume in read only mode.
-		 */
-		m_ui->checkBoxOpenReadOnly->setChecked( false ) ;
-		m_ui->checkBoxOpenReadOnly->setEnabled( false ) ;
-	}
-
 	m_reUseMountPoint = m_settings.reUseMountPoint() ;
 
 	m_checkBoxOriginalText = m_ui->checkBoxOpenReadOnly->text() ;
@@ -475,7 +465,7 @@ void keyDialog::setUpVolumeProperties( const keyDialog::entry& ee )
 	const auto& e      = ee.volEntry.favorite() ;
 	m_path             = e.volumePath ;
 	m_mountOptions     = e ;
-	m_favoriteReadOnly = e.readOnlyMode ;	
+	m_favoriteReadOnly = e.readOnlyMode ;
 
 	if( m_favoriteReadOnly.defined() ){
 
@@ -483,7 +473,9 @@ void keyDialog::setUpVolumeProperties( const keyDialog::entry& ee )
 	}else{
 		if( utility::platformIsWindows() ){
 
-			m_ui->checkBoxOpenReadOnly->setEnabled( false ) ;
+			m_windowsCanUnlockReadOnlyMode = ee.engine->windowsCanUnlocInReadWriteMode() ;
+
+			m_ui->checkBoxOpenReadOnly->setEnabled( m_windowsCanUnlockReadOnlyMode ) ;
 		}else{
 			m_ui->checkBoxOpenReadOnly->setEnabled( true ) ;
 			m_ui->checkBoxOpenReadOnly->setChecked( m_settings.getOpenVolumeReadOnlyOption() ) ;
@@ -675,11 +667,11 @@ void keyDialog::pbOptions()
 
 			m_checked = true ;
 
-			const auto& f = favorites::instance().readFavorite( m_path ) ;
+			auto f = favorites::instance().readFavorite( m_path ) ;
 
-			if( f.hasValue() ){
+			if( f.has_value() ){
 
-				m_mountOptions = f ;
+				m_mountOptions = f.value() ;
 			}
 		}
 
@@ -772,7 +764,7 @@ void keyDialog::textChanged( QString e )
 
 	m_ui->lineEditMountPoint->setText( e ) ;
 
-	auto r =  m_ui->lineEditFolderPath->text() ;
+	auto r = m_ui->lineEditFolderPath->text() ;
 
 	auto l = r.lastIndexOf( '/' ) ;
 
@@ -814,7 +806,7 @@ void keyDialog::enableAll()
 {
 	m_ui->pbMountPoint->setEnabled( true ) ;
 	m_ui->pbMountPoint_1->setEnabled( true ) ;
-        m_ui->pbOptions->setEnabled( true ) ;
+	m_ui->pbOptions->setEnabled( true ) ;
 	m_ui->label_2->setEnabled( true ) ;
 	m_ui->pbOpenFolderPath->setEnabled( true ) ;
 	m_ui->pbCancel->setEnabled( true ) ;
@@ -844,6 +836,8 @@ void keyDialog::enableAll()
 
 		m_ui->checkBoxOpenReadOnly->setEnabled( true ) ;
 	}else{
+		m_ui->checkBoxOpenReadOnly->setEnabled( m_windowsCanUnlockReadOnlyMode ) ;
+
 		m_ui->pbMountPoint_1->setEnabled( m_engine->supportsMountPathsOnWindows() ) ;
 	}
 
@@ -1011,7 +1005,7 @@ void keyDialog::pbOpen()
 			}
 		}else{
 			/*
-			 * We shouldnt get here
+			 * We shouldn't get here
 			 */
 			return this->openVolume() ;
 		}
@@ -1063,11 +1057,11 @@ void keyDialog::reportErrorMessage( const engines::engine::cmdStatus& s )
 
 		if( utility::platformIsWindows() ){
 
-			m_ui->checkBoxOpenReadOnly->setEnabled( false ) ;
+			m_ui->checkBoxOpenReadOnly->setEnabled( m_windowsCanUnlockReadOnlyMode ) ;
 
-			m_ui->checkBoxOpenReadOnly->setChecked( false ) ;
+			m_ui->checkBoxOpenReadOnly->setChecked( m_windowsCanUnlockReadOnlyMode ) ;
 		}
-	}	
+	}
 
 	this->showErrorMessage( s ) ;
 }
@@ -1442,7 +1436,7 @@ void keyDialog::pbSetKey()
 	auto keyFile    = m_ui->lineEditSetKeyKeyFile->text() ;
 	auto passphrase = m_ui->lineEditSetKeyPassword->text() ;
 
-	Task::run( [ = ](){
+	Task::run( [ this,keyFile,passphrase ](){
 
 		if( m_hmac ){
 
