@@ -112,7 +112,7 @@ bool utility::platformIsOSX()
 
 bool utility::platformIsWindows()
 {
-    return false ;
+	return false ;
 }
 
 #endif
@@ -164,14 +164,15 @@ utility::SocketPaths utility::socketPath()
 
 ::Task::future< utility::Task >& utility::Task::run( const QString& exe,
 						     const QStringList& list,
-						     bool e )
+						     const utility::Task::moreOptions& e )
 {
 	return utility::Task::run( exe,list,-1,e ) ;
 }
 
 ::Task::future< utility::Task >& utility::Task::run( const QString& exe,
 						     const QStringList& list,
-						     int s,bool e )
+						     int s,
+						     const utility::Task::moreOptions& e )
 {
 	return ::Task::run( [ = ](){
 
@@ -187,12 +188,11 @@ void utility::Task::execute( const QString& exe,
 			     const QProcessEnvironment& env,
 			     const QByteArray& password,
 			     std::function< void() > function,
-			     bool polkit,
-			     bool runs_in_background )
+			     const utility::Task::moreOptions& moreOpts )
 {
 	const auto& m = utility::miscOptions::instance() ;
 
-	if( polkit && m.usePolkit() ){
+	if( moreOpts.usePolkit() && m.usePolkit() ){
 
 		auto _report_error = [ this ]( const char * msg ){
 
@@ -230,6 +230,8 @@ void utility::Task::execute( const QString& exe,
 
 		logger.showText( exe,args ) ;
 
+		utility::unlockIntervalReporter rpt( moreOpts.likeSsh() ) ;
+
 		s.write( [ & ]()->QByteArray{
 
 			SirikaliJson json( logger.function() ) ;
@@ -246,6 +248,8 @@ void utility::Task::execute( const QString& exe,
 
 		s.waitForReadyRead() ;
 
+		rpt.report() ;
+
 		SirikaliJson json( s.readAll(),logger.function() ) ;
 
 		m_finished   = json.getBool( "finished" ) ;
@@ -256,50 +260,45 @@ void utility::Task::execute( const QString& exe,
 
 		logger.showText( { m_stdOut,m_stdError,m_exitCode,m_exitStatus,m_finished } ) ;
 	}else{
-		if( runs_in_background ){
+		using LG = utility2::LOGLEVEL ;
 
-			bool log = true ;
+		auto log = [ & ](){
 
-			if( exe.endsWith( "fscrypt" ) && args.size() > 0 && args.at( 0 ) == "status" ){
+			if( moreOpts.allowLogging().has_value() ){
 
-				log = false ;
-			}
-
-			utility::logger logger ;
-
-			if( log ){
-
-				logger.showText( exe,args ) ;
-			}
-
-			auto& ss = ::Task::process::run( exe,args,waitTime,password,env,std::move( function ) ) ;
-
-			auto s = utility::unwrap( ss ) ;
-
-			m_finished   = s.finished() ;
-			m_exitCode   = s.exit_code() ;
-			m_exitStatus = s.exit_status() ;
-			m_stdOut     = s.std_out() ;
-			m_stdError   = s.std_error() ;
-
-			if( log ){
-
-				logger.showText( s ) ;
-			}
-		}else{
-			utility::debug() << "Warning, Starting a detached process" ;
-
-			utility::logger().showText( exe,args ).showLine() ;
-
-			if( QProcess::startDetached( exe,args ) ){
-
-				m_exitCode = 0 ;
+				return moreOpts.allowLogging().value() ;
 			}else{
-				m_exitCode = 1 ;
+				return LG::COMMAND_ONLY ;
 			}
+		}() ;
 
-			m_finished   = true ;
-			m_exitStatus = 0 ;
+		utility::logger logger ;
+
+		if( log == LG::COMMAND_ONLY || log == LG::COMMAND_AND_UNLOCK_DURATION ){
+
+			logger.showText( exe,args ) ;
+		}
+
+		utility::unlockIntervalReporter rpt( moreOpts.likeSsh() ) ;
+
+		auto& ss = ::Task::process::run( exe,args,waitTime,password,env,std::move( function ) ) ;
+
+		auto s = utility::unwrap( ss ) ;
+
+		m_finished   = s.finished() ;
+		m_exitCode   = s.exit_code() ;
+		m_exitStatus = s.exit_status() ;
+		m_stdOut     = s.std_out() ;
+		m_stdError   = s.std_error() ;
+
+		if( log == LG::COMMAND_ONLY ){
+
+			logger.showText( s ) ;
+
+		}else if( log == LG::COMMAND_ONLY || log == LG::COMMAND_AND_UNLOCK_DURATION ){
+
+			rpt.report() ;
+			logger.showText( s ) ;
 		}
 	}
 }
@@ -540,6 +539,7 @@ options:\n\
 	     \"libsecret\" option causes SiriKali to read password from lxqt-wallet libsecret backend.\n\
 	     \"kwallet\" option causes SiriKali to read password from lxqt-wallet kwallet backend.\n\
 	     \"osxkeychain\" option causes SiriKali to read password from lxqt-wallet OSX key chain backend.\n\
+	     \"clargs\" option causes SiriKali to read password from cli argument given to option \"-w\"\
 	-k   When opening a volume from CLI,a value of \"rw\" will open the volume in read\\write\n\
 	     mode and a value of \"ro\" will open the volume in read only mode.\n\
 	-z   Full path of the mount point to be used when the volume is opened from CLI.\n\
@@ -1529,3 +1529,83 @@ QString utility::userIDAsString()
 {
 	return QString::number( utility::userID() ) ;
 }
+
+void utility::unlockIntervalReporter::report() const
+{
+	std::chrono::duration<double> m = this->currentTime() - m_origTime ;
+
+	auto s = [ & ](){
+
+		if( m_likeSsh.has_value() && m_likeSsh.value() ){
+
+			return "The attempt to connect took " ;
+		}else{
+			return "The attempt to unlock the volume took " ;
+		}
+	}() ;
+
+	utility::debug() << s + QString::number( m.count(),'f',2 ) + " seconds" ;
+}
+
+std::chrono::system_clock::time_point utility::unlockIntervalReporter::currentTime() const
+{
+	return std::chrono::system_clock::now() ;
+}
+
+QStringList utility::splitPreserveQuotes( const QString& e )
+{
+#if QT_VERSION < QT_VERSION_CHECK( 5,15,0 )
+	QStringList args ;
+	QString tmp ;
+	int quoteCount = 0 ;
+	bool inQuote = false ;
+
+	for( int i = 0 ; i < e.size() ; ++i ) {
+
+		const auto& s = e.at( i ) ;
+
+		if( s == '"' ){
+
+			quoteCount++ ;
+
+			if( quoteCount == 3 ) {
+
+				quoteCount = 0 ;
+				tmp.append( s ) ;
+			}
+
+			continue ;
+		}
+
+		if( quoteCount ){
+
+			if( quoteCount == 1 ){
+
+				inQuote = !inQuote ;
+			}
+
+			quoteCount = 0 ;
+		}
+
+		if( !inQuote && s.isSpace() ){
+
+			if( !tmp.isEmpty() ){
+
+				args.append( tmp ) ;
+				tmp.clear() ;
+			}
+		}else{
+			tmp.append( s ) ;
+		}
+	}
+
+	if( !tmp.isEmpty() ){
+
+		args.append( tmp ) ;
+	}
+
+	return args ;
+#else
+	return QProcess::splitCommand( e ) ;
+#endif
+}
